#include <LiquidCrystal.h>  //Librería para control del LCD
#include <IRremote.h>       //Libraría para control del Infrarrojo con mano Nec 
#include <SPI.h>            //Librería comunicación SPI (para el módulo RFID)
#include <RFID.h>           //Librería para control del módulo RFID
#include <EEPROM.h>         //Librería para utilizar la EEPROM interna del atmega328
#include <Stepper.h>        //Librería para control del motor paso a paso



 #define ESPERA 20      //Dirección Eeprom para guardar tiempo de espera
 #define SENSOR 21      //Dirección Eeprom para guardar decisión de uso de sensor
 #define PASS00 22      //Dirección Eeprom para guardar el primer dígito de la password
 #define PASS01 23      //Dirección Eeprom para guardar el segundo dígito de la password
 #define PASS02 24      //Dirección Eeprom para guardar el tercer dígito de la password
 #define PASS03 25      //Dirección Eeprom para guardar el cuarto dígito de la password
 #define PASS04 26      //Dirección Eeprom para guardar el quinto dígito de la password
 
 #define ultraPin 8     //Pin del sensor de ultrasonido
 #define led A5         //Pin del indicador luminoso
 
 #define numVueltas 1   //Vueltas necesarias para la maniobra de apertura/cierre

//LiquidCrystal lcd(RS, E, D4, D5, D6, D7);
LiquidCrystal lcd(6, 7, 5, 4, 3, 2);  //creamos el objecto lcd y asignamos los pines de comunicación


IRrecv irrecv(9);          //Creamos un objecto irrecv y le asignamos el pin 9
decode_results results;

RFID rfid(10, A4);        //Creamos un objecto rfide y le asignamos como pin de reset A4

const int stepsPerRevolution = 2048;        //Control 4 fases: 11,25º * 32 pasos * 64 reducción
Stepper myStepper(stepsPerRevolution, A0, A2, A1, A3);  //Creamos un objectos myStepper y le asignamos los pines de control



int modo = 0;    //Variable en la que almacenamos la posición del switch case (menu)
char maniobra;   //Variable en la que almacenamos el comando recibido por ir o la tarjeta
int tiempo = EEPROM.read(ESPERA);  //Inicializamos la variable tiempo con el valor de la EEPROM. Indica el tiempo de pausa en modo Automático
boolean sensor;  //Indica si usamos el sensor de ultrasonidos como seguridad en modo Automático o Manual
char passintro[6];  //Caracteres máximos Contraseña
int tarjeta = 0;    //Array para almacenar la contraseña introducida




void setup() {
  // put your setup code here, to run once:
  Serial.begin(9600);  //Inicializamos el puerto serie para mostrar lo que vamos haciendo
  
  irrecv.enableIRIn(); //Inicializamos el receptor infrarrojo
  
  lcd.begin(16, 2);    //Inicializamos el lcd indicando caracteres por línea y líneas.
  
  SPI.begin();   //Inicializamos la comunicación SPI
  
  rfid.init();  //Inicializamos el módulo RFID
    
  //defectoConfig();  //Función que almacena una configuración por defecto en la EEPROM
  
  verPass();  //Función que nos muestra por puerto serie cuál es la contraseña almacenada
  tarjetasAlmacenadas();  //Función que nos muestra por puerto serie las tarjetas almacenadas
  
   myStepper.setSpeed(5);  //Inicializamos el motor paso a paso
   
   if (EEPROM.read(SENSOR)==0xFF)  //Inicializamos la variable sensor según lo almacenado en la EEPROM
   sensor = true;
   else
   sensor = false;
   
   pinMode(led, OUTPUT);  //Inicializamos el pin del led indicador
  


}



//Función que comprueba si se ha recibido algo por el lector RFID o el mano Infrarrojo
//Cada vez que se lee una tarjeta, se comprueba que está habilitada; si lo está, pasa un comando 'A'
//Si no lo está, no pasa nada; sólo presenta en el display "Tarjeta fail"
//Comprueba si se ha recibido algo por infrarrojo
    void escucharOrden ()
    {
  
      if (rfidTarjeta ())
      {
      if(comprobarTarjeta (0)  || comprobarTarjeta (1) || comprobarTarjeta (2))
      {
      
      maniobra = 'A';
      Serial.println("Envia A");
      delay(500);
      }
      else
      {
      lcd.setCursor(0,2);
      lcd.print("Tarjeta fail");
      delay(1000);
      lcd.setCursor(0,2);
      lcd.print("                ");
      }
      }
    else
    {
    maniobra = consultarMando();
    }
    }



void loop() {
  // put your main code here, to run repeatedly:


//Función principal
//Son los diferentes puntos en los que se divide el programa
//Modos de funcionamiento y menú a través del lcd
    switch (modo)
    {
      
      case 0:            // En este modo sólo se necesita la orden de apertura desde mando IR o tarjeta proximidad
      {                  // El cierre es por tiempo
      lcd.clear();
      lcd.print ("PORTAL AUTOMAT"); 
      lcd.setCursor(0,2);
      indicarPortalDisplay();
      modo = 3;
      }break;
      
      case 1:            // En este modo es necesario comandar para abrir y cerrar bien con mando IR o tarjeta proximidad
      {                  // Permite parar el portal a medio recorrido
      lcd.clear();
      lcd.print ("PORTAL MANUAL");
      lcd.setCursor(0,2);
      indicarPortalDisplay();
      modo = 4;
      }break;
      
      case 2:            // Aquí podemos configurar algunos parámetros como el tiempo de espera a cierre o dar de alta tarjetas
      {
      lcd.clear();
      lcd.print ("PORTAL CONFIG");
      lcd.setCursor(0,2);
      lcd.print("PASS: ");
      if(password ())
      {    
      lcd.setCursor(7,2);  
      lcd.print("OK   ");
      delay(1000);
      modo = 5;
      }
      else
      {
      lcd.setCursor(7,2);
      lcd.print("FAIL ");
      modo = 0;
      }
      }break;
      
      case 3:          //Maniobra del modo Automático
      {
      escucharOrden ();
      if(maniobra == 'A')
      {
            
      if(!portal() || !fasePortal())
      {
      Serial.println("Abrir");
      maniobra = 0;
      abrirTodoStop();
      }
      
      while(portal())
      {
      espera();
      Serial.println("Cerrar");
      cerrarTodoStop();
      
      if(portal())
      {
      maniobra = 0;
      abrirTodoStop();
      }
      }

      }
      if(maniobra == 'M')
      {
      modo = 1;  
      }  
      }break;
      
      
      case 4:          //Maniobra del modo Manual
      {
      escucharOrden ();
      if(maniobra == 'A'|| maniobra == 'R')
      {
      if(!portal() || !fasePortal())
      {
      Serial.println("Abrir");
      maniobra = 0;
      abrirTodoStop();
      }
      
      while(portal()) 
      {
        escucharOrden ();
        if (maniobra == 'A'|| maniobra == 'R')
        {
        maniobra=0;
        Serial.println("Cerrar");
        cerrarTodoStop();
        }
      
 
      }
      
      }      
      else if(maniobra == 'M')
      {
      modo = 2;  
      }  
      }break;
      
      
      case 5:            //Dentro del menú de configuración
      {
      lcd.clear();
      lcd.print("CONFIG ON");
      lcd.setCursor(0,2);
      lcd.print("Espera: ");
      tiempo = EEPROM.read(ESPERA);
      modo = 6;
      }break;
      
      case 6:          //Tiempo espera a cierre en segundos
      {
      lcd.setCursor(8,2);
      lcd. print("   "); 
      lcd.setCursor(8,2);
      lcd. print(tiempo);
      delay(300);
      escucharOrden ();
       if(maniobra == 'A')
      {
        Serial.print("Aumentar tiempo");
        if(tiempo < 90)
        tiempo++;
        else
        tiempo=1;
        delay(100);
      }
      else if(maniobra == 'R')
      {
        Serial.print("Disminuir tiempo");
        if(tiempo > 1)
        tiempo--;
        else
        tiempo = 90;
        delay(100);
      }
      else if(maniobra == 'M')
      {
      Serial.print("Cambio menu");
      lcd.setCursor(0,2);
      lcd.print("Tarjetas BOR/ACT");
      tarjeta = 0;
      modo = 7;
      }
      }break;
      
      
      case 7:          //Gestión de tarjetas
      {
        escucharOrden ();
        if (maniobra == 'A')
        {
        lcd.setCursor(0,2);
        lcd.print("Actual Tarjetas ");
        delay(1000);
        modo = 8;        
        }
        else if (maniobra == 'R')    //Borrar todas las tarjetas
        {
        lcd.setCursor(0,2);
        lcd.print("Borrar Tarjetas ");
        borrarTarjetas();
        delay(1000);
        lcd.setCursor(0,2);
        lcd.print("Tarj BORRADAS   ");
        delay(1000);
        modo = 5;
        }
        else if (maniobra == 'M')
        {
        modo = 9;  
        }
        delay(300);
      }break;
      
      
      case 8:        //Memorizar tarjetas
      {
        escucharOrden ();
        if (maniobra == 'A')
        {
        if (tarjeta < 2)
        tarjeta ++;
        }
        else if (maniobra == 'R')
        {
        if (tarjeta > 0)
        tarjeta --;
        }
        else if (maniobra == 'M')
        {
        modo = 9;
        }
        
        lcd.setCursor(0,2);
        lcd.print("Tj");
        lcd.print(tarjeta);
        lcd.print(": ");
        imprimirTarjeta(tarjeta);
        delay(300);
        if(rfidTarjeta ())
        {
        lcd.setCursor(0,2);
        lcd.print ("Tarjeta ");
        lcd.print(tarjeta);
        lcd.print (": OK   ");
        guardarTarjeta(tarjeta);
        delay(2000);
        imprimirTarjeta(tarjeta);
        delay(1000);
        }
        
      }break;
      
      
      case 9:      //Habilitar o deshabilitar el uso del sensor de ultrasonidos como seguridad
      {
      if(EEPROM.read(SENSOR) == 0xFF)
      sensor = true;
      else
      sensor = false;
      modo = 10;
      }break;
      
      
      case 10:
      {
      lcd.setCursor(0,2);
      lcd.print("Sensor: ");
      lcd.setCursor(8,2);
      if(sensor)
      lcd.print("SI      ");
      else
      lcd.print("NO      ");
      escucharOrden ();
      if(maniobra == 'A')
      {
      sensor = true;
      Serial.print("Sensor true");
      }
      else if(maniobra == 'R')
      {
      sensor = false;
      Serial.print("Sensor false");
      }
      else if(maniobra == 'M')
      {
      lcd.setCursor(0,2);
      lcd.print("Cambiar Pass    ");
      modo = 11;
      } 
      delay(300);   
      }break;
      
      
            
      case 11:      //Cambio de contraseña
      {
      escucharOrden ();
      if(maniobra == 'A')
      {
      lcd.setCursor(0,2);
      lcd.print("Nueva:          "); 
      lcd.setCursor(7,2);
      int a = 0;
      while (a < 5 )
      {
      while(!irrecv.decode(&results));
      passintro[a] = consultarMando();
      if (passintro[a] == 'M')
      {
      a = 10;
      }
      lcd.print(passintro[a]);
      a++;
      }
      lcd.print(" OK");
      }
      else if(maniobra == 'M')
      {
      lcd.setCursor(0,2);
      lcd.print("Guardar NO/SI   ");
      delay(2000);
      modo = 12;
      }
      }break;
      
      case 12:        //Guardar en EEPROM el tiempo de espera, el uso o no del sensor y la password
      {
      escucharOrden ();
      if(maniobra == 'R')
      {
      lcd.setCursor(0,2);
      lcd.print("Guardar: NO     ");
      delay(2000);
      modo = 0;
      }
      else if(maniobra == 'A')
      {
      lcd.setCursor(0,2);
      lcd.print("Guardar: Si    ");
      delay(1000);
      lcd.setCursor(0,2);
      lcd.print("Guardando ...  ");
      EEPROM.write(ESPERA, tiempo);
      if(sensor == true)
      EEPROM.write(SENSOR, 0xFF);
      else
      EEPROM.write(SENSOR, 0x00);
      guardarPass();
      delay(2000);
      modo = 0;
      } 
      else if(maniobra == 'M')
      {
      modo = 5;
      }
      }break;
      
    
      
    }  
   
}
